@@ -17,6 +17,8 @@
 
 #define warn(...) fprintf(stderr, __VA_ARGS__)
 
+static volatile sig_atomic_t exiting = 0;
+
 const char *argp_program_version = "tcpconnect 0.1";
 const char *argp_program_bug_address =
 	"https://github.com/iovisor/bcc/tree/master/libbpf-tools";
@@ -184,19 +186,16 @@ static error_t parse_arg(int key, char *arg, struct argp_state *state)
 	return 0;
 }
 
-static int libbpf_print_fn(enum libbpf_print_level level,
-		const char *format, va_list args)
+static int libbpf_print_fn(enum libbpf_print_level level, const char *format, va_list args)
 {
 	if (level == LIBBPF_DEBUG && !env.verbose)
 		return 0;
 	return vfprintf(stderr, format, args);
 }
 
-static volatile sig_atomic_t hang_on = 1;
-
 static void sig_int(int signo)
 {
-	hang_on = 0;
+	exiting = 1;
 }
 
 static void print_count_ipv4(int map_fd)
@@ -261,7 +260,7 @@ static void print_count(int map_fd_ipv4, int map_fd_ipv6)
 {
 	static const char *header_fmt = "\n%-25s %-25s %-20s %-10s\n";
 
-	while (hang_on)
+	while (!exiting)
 		pause();
 
 	printf(header_fmt, "LADDR", "RADDR", "RPORT", "CONNECTS");
@@ -325,28 +324,26 @@ static void handle_lost_events(void *ctx, int cpu, __u64 lost_cnt)
 
 static void print_events(int perf_map_fd)
 {
-	struct perf_buffer_opts pb_opts = {
-		.sample_cb = handle_event,
-		.lost_cb = handle_lost_events,
-	};
-	struct perf_buffer *pb = NULL;
+	struct perf_buffer *pb;
 	int err;
 
-	pb = perf_buffer__new(perf_map_fd, 128, &pb_opts);
-	err = libbpf_get_error(pb);
-	if (err) {
-		pb = NULL;
+	pb = perf_buffer__new(perf_map_fd, 128,
+			      handle_event, handle_lost_events, NULL, NULL);
+	if (!pb) {
+		err = -errno;
 		warn("failed to open perf buffer: %d\n", err);
 		goto cleanup;
 	}
 
 	print_events_header();
-	while (hang_on) {
+	while (!exiting) {
 		err = perf_buffer__poll(pb, 100);
-		if (err < 0 && errno != EINTR) {
-			warn("Error polling perf buffer: %d\n", err);
+		if (err < 0 && err != -EINTR) {
+			warn("error polling perf buffer: %s\n", strerror(-err));
 			goto cleanup;
 		}
+		/* reset err to return 0 if exiting */
+		err = 0;
 	}
 
 cleanup:
@@ -368,14 +365,9 @@ int main(int argc, char **argv)
 	if (err)
 		return err;
 
+	libbpf_set_strict_mode(LIBBPF_STRICT_ALL);
 	libbpf_set_print(libbpf_print_fn);
 
-	err = bump_memlock_rlimit();
-	if (err) {
-		warn("failed to increase rlimit: %s\n", strerror(errno));
-		return 1;
-	}
-
 	obj = tcpconnect_bpf__open();
 	if (!obj) {
 		warn("failed to open BPF object\n");
@@ -408,7 +400,8 @@ int main(int argc, char **argv)
 	}
 
 	if (signal(SIGINT, sig_int) == SIG_ERR) {
-		warn("can't set signal handler: %s\n", strerror(-errno));
+		warn("can't set signal handler: %s\n", strerror(errno));
+		err = 1;
 		goto cleanup;
 	}
 
