@@ -46,7 +46,7 @@ int test(struct bpf_sock_ops *skops)
     ebpf::BPF bpf;
     ebpf::StatusTuple res(0);
     res = bpf.init(BPF_PROGRAM);
-    REQUIRE(res.code() == 0);
+    REQUIRE(res.ok());
 
     // create a udp socket so we can do some map operations.
     int sockfd = socket(AF_INET, SOCK_DGRAM, 0);
@@ -56,11 +56,11 @@ int test(struct bpf_sock_ops *skops)
     int key = 0, val = sockfd;
 
     res = sk_map.remove_value(key);
-    REQUIRE(res.code() != 0);
+    REQUIRE(!res.ok());
 
     // the socket must be TCP established socket.
     res = sk_map.update_value(key, val);
-    REQUIRE(res.code() != 0);
+    REQUIRE(!res.ok());
   }
 }
 
@@ -89,7 +89,7 @@ int test(struct bpf_sock_ops *skops)
     ebpf::BPF bpf;
     ebpf::StatusTuple res(0);
     res = bpf.init(BPF_PROGRAM);
-    REQUIRE(res.code() == 0);
+    REQUIRE(res.ok());
 
     // create a udp socket so we can do some map operations.
     int sockfd = socket(AF_INET, SOCK_DGRAM, 0);
@@ -99,11 +99,11 @@ int test(struct bpf_sock_ops *skops)
     int key = 0, val = sockfd;
 
     res = sk_hash.remove_value(key);
-    REQUIRE(res.code() != 0);
+    REQUIRE(!res.ok());
 
     // the socket must be TCP established socket.
     res = sk_hash.update_value(key, val);
-    REQUIRE(res.code() != 0);
+    REQUIRE(!res.ok());
   }
 }
 
